@@ -29,7 +29,6 @@
 #include "Geolocation.h"
 
 #include "Chrome.h"
-#include "Document.h"
 #include "Frame.h"
 #include "Page.h"
 #include <wtf/CurrentTime.h>
@@ -220,6 +219,7 @@ Geolocation::~Geolocation()
 
 void Geolocation::disconnectFrame()
 {
+    stopTimers();
     stopUpdating();
     if (m_frame) {
         if (m_frame->document())
@@ -232,6 +232,8 @@ void Geolocation::disconnectFrame()
 
 Geoposition* Geolocation::lastPosition()
 {
+    ASSERT(isAllowed());
+
 #if ENABLE(CLIENT_BASED_GEOLOCATION)
     if (!m_frame)
         return 0;
@@ -379,22 +381,6 @@ void Geolocation::clearWatch(int watchId)
         stopUpdating();
 }
 
-void Geolocation::suspend()
-{
-#if !ENABLE(CLIENT_BASED_GEOLOCATION)
-    if (hasListeners())
-        m_service->suspend();
-#endif
-}
-
-void Geolocation::resume()
-{
-#if !ENABLE(CLIENT_BASED_GEOLOCATION)
-    if (hasListeners())
-        m_service->resume();
-#endif
-}
-
 void Geolocation::setIsAllowed(bool allowed)
 {
     // This may be due to either a new position from the service, or a cached
